@@ -11,6 +11,8 @@ import json
 from typing import Dict, List, Optional, Any, Tuple
 from pathlib import Path
 import logging
+
+from .json_utils import safe_json_dump, safe_json_serialize
 try:
     from PIL import Image, ImageDraw, ImageChops
     import numpy as np
@@ -107,9 +109,14 @@ class MockupComparator:
                     await mockup_browser.set_viewport(viewport["width"], viewport["height"])
                     await implementation_browser.set_viewport(viewport["width"], viewport["height"])
                     
-                    # Navigate to initial pages
+                    # Navigate to initial pages with stabilization
                     await mockup_browser.navigate("/")
+                    await asyncio.sleep(1)
+                    await mockup_browser.page.wait_for_load_state("domcontentloaded")
+                    
                     await implementation_browser.navigate("/")
+                    await asyncio.sleep(1)
+                    await implementation_browser.page.wait_for_load_state("domcontentloaded")
                     
                     # Execute any required actions on mockup
                     if mockup_actions:
@@ -119,6 +126,9 @@ class MockupComparator:
                     if implementation_actions:
                         await self._execute_actions_on_browser(implementation_browser, implementation_actions)
                     
+                    # Final stabilization before capturing
+                    await asyncio.sleep(0.5)
+                    
                     # Capture screenshots
                     mockup_screenshot = await self._capture_comparison_screenshot(
                         mockup_browser, f"{comparison_name}_mockup_{viewport_name}"
@@ -163,19 +173,12 @@ class MockupComparator:
                     "implementation_url": implementation_url,
                     "viewports_tested": len(viewports),
                     "results": comparison_results,
-                    "summary": self._create_comparison_summary(comparison_results),
-                    "recommendations": self._generate_improvement_recommendations(comparison_results)
+                    "summary": self._create_comparison_summary(comparison_results)
                 }
                 
-                # Save comparison report
+                # Save comparison report with safe serialization
                 report_path = self.artifacts_base / "mockup_comparisons" / f"{comparison_name}.json"
-                with open(report_path, 'w') as f:
-                    # Custom JSON encoder to handle numpy booleans/integers
-                    def json_serializer(obj):
-                        if hasattr(obj, 'item'):  # numpy types
-                            return obj.item()
-                        return str(obj)
-                    json.dump(comparison_report, f, indent=2, default=json_serializer)
+                safe_json_dump(comparison_report, str(report_path))
                 
                 self.logger.info(f"Mockup comparison completed: {comparison_name}")
                 return comparison_report
@@ -293,10 +296,9 @@ class MockupComparator:
                     "final_recommendations": self._generate_final_recommendations(iteration_results)
                 }
                 
-                # Save iteration report
+                # Save iteration report with safe JSON serialization
                 report_path = self.artifacts_base / "iteration_progress" / f"{iteration_session_id}.json"
-                with open(report_path, 'w') as f:
-                    json.dump(iteration_report, f, indent=2, default=str)
+                safe_json_dump(iteration_report, str(report_path))
                 
                 self.logger.info(f"UI matching iteration completed: {iteration_session_id}")
                 return iteration_report
@@ -314,11 +316,15 @@ class MockupComparator:
             if "navigate" in action:
                 path = action["navigate"]
                 await browser.navigate(path)
+                # Wait for page to stabilize after navigation
+                await asyncio.sleep(1)  # Brief pause for dynamic content rendering
+                await browser.page.wait_for_load_state("domcontentloaded")
             elif "click" in action:
                 selector = action["click"]
                 if isinstance(selector, dict):
                     selector = selector["selector"]
                 await browser.click(selector)
+                await asyncio.sleep(0.5)  # Brief pause for UI response
             elif "wait_for" in action:
                 selector = action["wait_for"]
                 if isinstance(selector, dict):
@@ -329,6 +335,7 @@ class MockupComparator:
             elif "scroll" in action:
                 scroll_config = action["scroll"]
                 await browser.page.evaluate(f"window.scrollTo({scroll_config.get('x', 0)}, {scroll_config.get('y', 0)})")
+                await asyncio.sleep(0.3)  # Brief pause for scroll rendering
     
     async def _capture_comparison_screenshot(self, browser: BrowserController, name: str) -> str:
         """Capture screenshot for comparison"""
@@ -1223,55 +1230,19 @@ class MockupComparator:
         
         avg_similarity = sum(similarities) / len(similarities) if similarities else 0
         
+        # Pure data summary - no interpretation
         return {
             "average_similarity": round(float(avg_similarity), 2),
             "viewports_tested": len(comparison_results),
-            "best_viewport_match": max(comparison_results, key=lambda x: x.get("visual_diff", {}).get("similarity_score", 0)),
-            "needs_improvement": bool(avg_similarity < 80)  # Convert numpy.bool_ to Python bool
+            "similarity_by_viewport": [
+                {
+                    "viewport": result.get("viewport", {}).get("name", "unknown"),
+                    "similarity": float(result.get("visual_diff", {}).get("similarity_score", 0))
+                }
+                for result in comparison_results
+            ]
         }
     
-    def _generate_improvement_recommendations(self, comparison_results: List[Dict]) -> List[Dict]:
-        """Generate recommendations for improving implementation to match mockup"""
-        recommendations = []
-        
-        for result in comparison_results:
-            viewport = result.get("viewport", {})
-            layout_analysis = result.get("layout_analysis", {})
-            visual_diff = result.get("visual_diff", {})
-            
-            # Analyze layout differences for recommendations
-            differences = layout_analysis.get("differences", [])
-            
-            for diff in differences:
-                if diff["type"] == "missing_in_implementation":
-                    recommendations.append({
-                        "type": "add_element",
-                        "priority": "high",
-                        "description": f"Add missing element: {diff['selector']}",
-                        "viewport": viewport.get("name", "unknown")
-                    })
-                elif diff["type"] == "property_differences":
-                    for prop, prop_diff in diff.get("differences", {}).items():
-                        recommendations.append({
-                            "type": "adjust_property",
-                            "priority": "medium",
-                            "description": f"Adjust {prop} for {diff['selector']}",
-                            "current_value": prop_diff.get("implementation"),
-                            "target_value": prop_diff.get("mockup"),
-                            "viewport": viewport.get("name", "unknown")
-                        })
-            
-            # Add visual similarity recommendations
-            similarity = visual_diff.get("similarity_score", 0)
-            if similarity < 70:
-                recommendations.append({
-                    "type": "major_visual_changes",
-                    "priority": "high",
-                    "description": f"Significant visual differences detected (similarity: {similarity}%)",
-                    "viewport": viewport.get("name", "unknown")
-                })
-        
-        return recommendations
     
     def _create_iteration_summary(self, baseline_comparison: Dict, iteration_results: List[Dict]) -> Dict[str, Any]:
         """Create summary of iteration session"""
